#include "AD9833.hpp"
#include "systemUtil.hpp"
#include "stringUtil.hpp"
#include "string.h"
#include "stdio.h"

#define SPI_Speed   1000000 //alt 12500000

#define DDS_MIN_FREQ	  		 25	// DDS minimale Frequenz 
#define DDS_MAX_FREQ	  10000000	// DDS maximale Frequenz 

/* DDS control word	*/
#define AD9833_B28			0x2000
#define AD9833_RESET		0x0100
#define AD9833_OPBITEN	0x0020
#define AD9833_DIV2			0x0008
#define AD9833_MODE			0x0002
#define AD9833_FSELECT	0x0800


//#define AD9833_INPUT_CLOCK	12000000	/*! DDS Inputclock (12MHz)			*/
#define AD9833_MIN_FREQ	    100	/*! DDS minimale Frequenz (100Hz)	*/
#define AD9833_MAX_FREQ			10000000	/*! DDS maximale Frequenz (10MHz)	*/


#define SETFREQREG() (( m_bFreqReg1 ? AD9833_FSELECT : 0 ))
// AD9833_B28 ist der Default Zustand, es wird Freq0 /1  und Phase0 benutzt

// pCSPort = NULL -> es wird der CS-Pin benutzt der dem SPI-Bus zugeordnet ist 
// pCSPort != NULL -> der angegebene Pin wird fr den CS benutzt
CAD9833::CAD9833( CSpi *SpiBus, uint32_t InputFrequenz, CPinOut *pCSPort, char * Text, unsigned int Len, CMonitor *pMonitor ):
								CMonitorBef( Text, Len, pMonitor, false ) 
{
	m_pSpiBus = SpiBus;
	
	m_LastWave = INIT_WAVE;
	
	m_WaveRegBits = 0;
	
	m_InputFrequenz = InputFrequenz;
	
	m_pCSPort = pCSPort;

	m_bFreqReg1 = false;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CAD9833::Init( )
{
	Reset();
	Signalform( WAVE_SINUS );
	Phase( 0 );
	return true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CAD9833::Reset( )
{
	uint16_t Reg = 0;
	
	SetWaveReg( WAVE_SINUS, Reg );
	
	Reg |= AD9833_RESET;
	WriteReg( &Reg, 1 );
	
	m_SysUtil.WaitMs( 1 );
	
	Reg &= ~AD9833_RESET;
	Reg |= AD9833_B28 | SETFREQREG() | m_WaveRegBits;		//Default 2 Word Zugriff
	WriteReg( &Reg, 1 );
	
	return true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CAD9833::SetWaveReg( enumWave Wave, uint16_t &Reg )
{
	if( Wave == WAVE_NOCHANGE )
		Wave = m_LastWave;

	switch( Wave )
	{
		case WAVE_NOCHANGE:
			break;
		case WAVE_SINUS:
			Reg &= ~( AD9833_OPBITEN | AD9833_DIV2 | AD9833_MODE );
			break;
		case WAVE_SQUARE:
			Reg &= ~AD9833_MODE;
			Reg |= AD9833_OPBITEN | AD9833_DIV2;
			break;
		case WAVE_TRIANGLE:
			Reg &= ~( AD9833_OPBITEN | AD9833_DIV2 );
			Reg |= AD9833_MODE;
			break;
		case INIT_WAVE:
		default:
			return false;
	}
	Reg |= AD9833_B28;		//Default 2 Word Zugriff
	
	m_WaveRegBits = Reg;
	m_LastWave = Wave;
	return true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CAD9833::WriteReg( uint16_t *Reg, uint8_t Count )
{
	bool bRet = false;
	
	bRet = m_pSpiBus->Open( SPI_Speed, SSP_BITS_16, SSP_CLOCK_CPHA0_CPOL1, m_pCSPort );
	if( bRet )
			bRet = m_pSpiBus->WriteCs( (uint8_t*)Reg, Count * 2 );

	m_pSpiBus->Close( );

	return bRet;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CAD9833::Signalform( enumWave Wave )
{
	uint16_t Reg[5];
	Reg[0] = 0;

	// Signalform setzen
	if( !SetWaveReg( Wave, Reg[0] ) )
		return false;

	Reg[0] |= SETFREQREG() | m_WaveRegBits;
	// Register belegen
	return WriteReg( Reg, 1 );
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CAD9833::Phase( uint16_t Phase )
{
	uint16_t Reg[5];
	Reg[0] = 0;

	uint16_t PhasenReg = Phase;
	Reg[0] = ( PhasenReg & 0x0fff ) | 0xC000;  

	// Register belegen
	return WriteReg( Reg, 1 );
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CAD9833::Frequenz( uint32_t Frequenz )
{
	uint16_t Reg[5];

	// ist die gewnschte Frequenz zulssig
	if( Frequenz > DDS_MAX_FREQ )
		Frequenz = DDS_MAX_FREQ;
	
	if( Frequenz < DDS_MIN_FREQ )
		Frequenz = DDS_MIN_FREQ;

	
//			                hz *  2^28
//			freqReg = -----------------------
//			               DDS_INPUT_CLOCK
	uint64_t freqReg = 0x10000000 / 4;
	freqReg *= Frequenz;
	freqReg /= m_InputFrequenz / 4;

//	Reg[0] = 0;
	// Signalform setzen
//	if( !SetWaveReg( Wave, Reg[0] ) )
//		return false;
	
	// Frequenz wird mit zwei Zugriffen gesetzt
//	Reg[0] |= AD9833_B28;
	
	// Frequenzregister 0 setzen
	Reg[0] = (uint16_t)( ( freqReg & 0x3fff ) | ( m_bFreqReg1 ? 0x8000 : 0x4000 ) );  // LSB
	Reg[1] = (uint16_t)( ( ( freqReg >> 14 ) & 0x3fff ) | ( m_bFreqReg1 ? 0x8000 : 0x4000 ) );  // MSB
	Reg[2] = SETFREQREG() | m_WaveRegBits;  
	
	m_bFreqReg1 = ~m_bFreqReg1;
	
	// Phasenregister 0 setzen
//	uint16_t PhasenReg = 0;
//	Reg[3] = ( PhasenReg & 0x0fff ) | 0xC000;  
	
	// Register belegen
	return WriteReg( Reg, 3 );
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CAD9833::Work( char *Text, unsigned int &Len )
{
	int Index, WordLen;

	if( ( Index = CStringUtil::Scan( Text, Len, WordLen ) ) == -1 )
	  // keine weiteren Eingabe -> Fehleingabe
		return false;
	else
	{  // weitere Eingaben auswerten
		// Befehl zu lang -> Fehleingabe
		if( WordLen != 1 )
			return false;

		int Wert;
		// Befehl ausfhren
		switch( Text[Index] )
		{
			case 'F':
			case 'f':
				// Frequenz setzen
				// Wert holen
				Index = CStringUtil::Scan( Text, Len, WordLen, Index );
				if( Index == -1 )
					return false;
				if( !CStringUtil::ScanDez( &Text[Index], WordLen, &Wert ) )
					return false;
				return Frequenz( Wert );
			case 'S':
			case 's':
				// Sinus
				return Signalform( WAVE_SINUS );
			case 'D':
			case 'd':
				// Dreieck
				return Signalform( WAVE_TRIANGLE );
			case 'R':
			case 'r':
				// Rechteck
				return Signalform( WAVE_SQUARE );
			default:
				return false;
		}
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CAD9833::ShortHelp()
{
	m_pMonitor->Send( (char*)"DDS Frequenz und Signalform aendern\n" );
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CAD9833::LongHelp()
{
	CMonitorBef::LongHelp( (char*)"DDS Frequenz und Signalform aendern\nSyntax: %s F Frequenz  - Frequenz [Hz] aendern\n" );
	m_pMonitor->Send( (char*)     	                                   "           S           - Signalform Sinus einstellen\n" );
	m_pMonitor->Send( (char*)                                          "           R           - Signalform Rechteck einstellen\n" );
	m_pMonitor->Send( (char*)                                          "           D           - Signalform Dreieck einstellen\n" );
}

